/* ----------------------------------------------------------------------------
 *  gpsWrapper
 * ----------------------------------------------------------------------------
 *  just a wrapper class around TinyGps + some Functions from Wirering NMEA-Lib
 * ----------------------------------------------------------------------------
 *  prog: max.rheiner@zhdk.ch
 * ----------------------------------------------------------------------------
 */
 
#ifndef _GPS_WRAPPER_H
#define _GPS_WRAPPER_H

#include <WProgram.h>
#include <TinyGPS.h>

class GpsWrapper
{
public:

  GpsWrapper():
  _serial(NULL)
  {;}
  
  bool start(HardwareSerial* serialPort,int baudRate);
  void end();
  
  // reads in the serial data
  bool parseGps();
  
  // poisition
  long           latitude(){ return _lat;}
  long           longitude(){ return _long;}
  float          latitudef(){ return _lat * 1.0f / 100000.0f;}
  float          longitudef(){ return _long * 1.0f / 100000.0f;}
  unsigned long  fixAge() { return _fix; }
  
  float          altitude() {return _gps.altitude() * 1.0f/ 100.0f;}
  float          course(){return _gps.course() * 1.0f/ 100.0f;}
  float          speedKmph(){return _gps.f_speed_kmph();} 
  
  // time
  int            year(){ return _year;}
  byte           month(){ return _month;}
  byte           day(){ return _day;}
  byte           hour(){ return _hour;}
  byte           minute(){ return _minute;}
  byte           second(){ return _second;}
  byte           hundredths(){ return _hundredths;}
 
  
  bool           isValid() { return _valid; }
  TinyGPS*       gpsObj() { return &_gps; }
  
  // prints out the gpsData to the serialPort
  void           dump(HardwareSerial* serialPort);
  
  // static functions
  static float   distance(float lat1, float long1, float lat2, float long2, float unitsPerMeter);
  static float   angle(float lat1, float long1, float lat2, float long2);
  
  static void    printFloat(float number, int digits=2);
  
protected:
  
  void updateData();
  
  bool              _valid;
  TinyGPS           _gps;
  HardwareSerial*   _serial;
  
  long              _lat;
  long              _long;
  
  unsigned long     _fix;
  
  int               _year;
  byte              _month;
  byte              _day;
  byte              _hour;
  byte              _minute;
  byte              _second;
  byte              _hundredths;
};
 
 
#endif // _GPS_WRAPPER_H
